#!/bin/bash
###############################################################################
# (C) Copyright 2013-2014 Hewlett-Packard Development Company, L.P.
# @(#) PPAS Toolkit Modular package configuration script.
# @(#) Product Name                :  HP Serviceguard Toolkit for PPAS.
# @(#) Product Version             :  A.12.00.00
#
# *** Note: This file MUST NOT be edited. ***
#
# Any changes made to this file will be overwritten when you upgrade
# to the next release of PPAS.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
#-----------------------------------------------------------------------
# Description:  PPAS Toolkit Modular package configuration script.
#
# The existing package control script is modularized and divided into
# 3 types of individual scripts:
#
# (1) The master control script, which uses the package configuration
#     data retrieved via 'cmgetpkgenv' or a temporary environment
#     file to determine what actions to take and in what order.
#     Specifically, the "OPERATION_SEQUENCE[*]" attributes from
#     the package configuration data controls the module script
#     execution order.
#
# (2) The action scripts (aka module scripts), which are associated
#     with a Attribute Definition File (ADF) and perform module
#     specific actions using the configured attributes defined in ADF.
#
# (3) The utility functions, which are used by the module scripts for
#     rudimentary functions such as logging and tracing of the messages
#     to ensure an uniform package logging format.
#
# Input : The script accepts two parameters: op_code and filename.
#
#    op_code :  The first argument is the operation to be performed whose
#               valid values are: start, stop, ppas_monitor, validate,
#               reconfigure_start and reconfigure_stop.
#               "start":     Calls PPAS toolkit scripts with 'start' option.
#                            If error occurs, execute each previous module
#                            script with stop parameter in reverse order.
#               "stop" :     Calls each of the module scripts with 'stop'
#                            option.
#               "ppas_monitor": Calls PPAS toolkit scripts with 'monitor' option.
#               "validate":  Calls each of the module scripts with 'validate'
#                            option.
#               "reconfigure_start": Calls each of the module scripts with
#                            'reconfigure_start' option
#               "reconfigure_stop": Calls each of the module scripts with
#                            'reconfigure_stop' option
#
# Output : Errors/Log messages. 
#          In case of "validate" the messages are printed to stdout.
#          For "start" and "stop" cases the messages are directed to
#          $SG_SCRIPT_LOG_FILE. If the SCRIPT_LOG_FILE is not specified
#          the messages go to ${SGRUN}/log/${SG_PACKAGE_NAME}.log. 
#
# Exit:     0  When the execution of all the scripts is successful.
#           1  When there is an error condition from the script and the
#              package should not be restarted as a result of the error.
#           2  When there is an error condition from the script, but the
#              package can be restarted on an alternate node.
#
#########################################################################

#
# Source in Serviceguard environment variables
#
. /etc/cmcluster.conf

#
# Source in the utility functions used by the module scripts
#
if [[ -z $SG_UTILS ]]
then
    SG_UTILS=$SGCONF/scripts/mscripts/utils.sh
fi
if [[ -f ${SG_UTILS} ]]
then
    . ${SG_UTILS}
    if [[ $? -ne 0 ]]
    then
        echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
    exit 1
fi

#
# PATH for Linux
#
PATH=$SGSBIN:/usr/bin:/usr/sbin:/etc:/bin:/sbin:/usr/lbin
export PATH

#
# Get the environment for this package through utility function
# sg_source_pkg_env().
#
sg_source_pkg_env $*
 
typeset -r TKIT_SCRIPT_DIR=$SGROOT/ppastoolkit

#
# Get parameters
#
if [[ $# == 0 ]]
then
   sg_log 0 "ERROR: No parameter is passed to the command."
   sg_exit 1 $*
fi 

#
# Export the global variable op_code  
#
typeset -x op_code="${1}"

typeset -i exit_val=0

SLEEP=/bin/sleep
KILL=/bin/kill
PS=/bin/ps
ECHO=/bin/echo
CUT=/usr/bin/cut
GREP=/bin/grep
AWK=/bin/awk
SU=/bin/su
HOST=/bin/hostname
DATE=/bin/date
BASH=/bin/bash
HEAD=/usr/bin/head
PKG_DIR=${0%/*}
CONF_DIR=${PPAS_TKIT_DIR:=$PKG_DIR}
ID=/usr/bin/id
###############################################################################
# Function: validate_function
#
# Validate if the PPAS package can run on the package nodes.
# Return 1 if at least one validation failed, otherwise return 0.
###############################################################################
function validate_function
{
   sg_log 5 "PPAS Toolkit validation" 
   typeset -i return_val=0
    # Fail the online pkg reconfiguration if unsupported pkg attributes are modified.
    # Check if a new package configuration is being applied or an existing package 
    # configuration is being changed. 
     if  [[ -z "$SG_PRIV_CLUSTER_VERIFICATION" ]] &&
         [[ "x$PREV_PPAS_DB_DATA_DIR" != "x" ]]
     then
        cmviewcl -p ${SG_PACKAGE_NAME} | $GREP running >> /dev/null 2>&1
        if [[ $? -eq 0 ]] && [[ "${PREV_SG_MODULE_NAME[@]}" = *tkit/ppas/ppas* ]]
    	then
           verify_unsupported_online_modification
           if [[ $? -eq 1 ]]; then 
              return 1
           fi
        fi
     fi 
  
   # Check if the PPAS administrator is present on the package nodes 
   $ID $PPAS_DB_USER >> /dev/null 2>&1
    if [[ $? -ne 0 ]]; then  
        sg_log 0 "ERROR: PPAS administrator user "$PPAS_DB_USER" is not present on node $($HOST)"
        return_val=1 
    fi
  
   # Checks for  cmcheckconf -v command
    if [[ "$SG_PRIV_CLUSTER_VERIFICATION" = "y" ]]; then  
       # Check if the required PPAS binary is present or not 
       if [ ! -f $PPAS_DB_HOME/bin/pg_ctl ] 
       then
           sg_log 0 "INFO: Could not find the required PPAS binary pg_ctl on node $($HOST)"
       else
        # Check if the installed version of PPAS is supported or not
           version1=`$PPAS_DB_HOME/bin/pg_ctl -V | $CUT  -d " " -f 3` >> /dev/null 2>&1
           version2=`$ECHO $version1 | $CUT  -d "." -f 1` >> /dev/null 2>&1
           if [[ $version2 -lt 9 ]] ;then
              sg_log 0 "INFO: PPAS toolkit do not support PPAS version $version1 on $($HOST)"
           fi
        fi
     fi
    
     return $return_val
 
}

##########################################################################
# Verify whether unsupported toolkit attributes are being changed during
# online reconfiguration. If any of the unsupported attributes are being
# modified then function will return 1 (failure) else return 0 (success).
##########################################################################

function verify_unsupported_online_modification 
{
    typeset -i flag=0
    typeset -i i=0 
    # These arrays define following things 
    # curarrayname => Attribute Names for which online modifcation is not supported  
    # cuarray => Current values of the unsupported attribitues 
    # prevarray => Previous values of unsupported attribtes 
    declare -a curarrayname=( "DB_DATA_DIR" "DB_LOG_FILE" "DB_HOME" "DB_USER" ) 
    declare -a curarray=( "$PPAS_DB_DATA_DIR" "$PPAS_DB_LOG_FILE" "$PPAS_DB_HOME" "$PPAS_DB_USER" ) 
    declare -a prevarray=( "$PREV_PPAS_DB_DATA_DIR" "$PREV_PPAS_DB_LOG_FILE" "$PREV_PPAS_DB_HOME" "$PREV_PPAS_DB_USER" ) 
    while (( $i <= ${#curarray[@]} -1 )) 
    do 
        if [[ "${curarray[i]}" != "${prevarray[i]}" ]]; then 
                 sg_log 0 "ERROR: Online Modification of attribute ${curarrayname[i]} value is not supported" 
                 flag=1  
        fi 
        (( i = i + 1 )) 
    done 
    if [[ $flag -eq 1 ]]; then 
       return 1 
    fi 
    return 0   
}


###############################################################################
# Function: check_return
#
# Check the return value of functions and print appropriate message
# if the return value is not equals to "0". Set exit_val with "1" in
# case of fatal error.
# It accepts two parameters.
# Parameters:
# 1. Exit value of the previously executed command
# 2. The place of execution
#        1 - ppas_dbi_start_cmd
#        2 - ppas_dbi_stop_cmd
#        3 - Monitor script
#        4 - validate_function
#        5 - reconfigure_start_function
#        6 - Improper parameters to toolkit script
###############################################################################
function check_return
{
    if [[ $1 -ne 0 ]]
    then
        case $2 in
            1)
                sg_log 0 "ERROR: Function ppas_dbi_start_cmd: Failed to start PPAS Instance."
                exit_val=1
                ;;
            2)
                sg_log 0 "ERROR: Function ppas_dbi_stop_cmd: Failed to stop PPAS Instance."
                exit_val=1
                ;;
            3)
                sg_log 0 "ERROR: PPAS monitoring failed. Package will be halted."
                exit_val=1
                ;;
            4)
                sg_log 0 "ERROR: Function validate_function: PPAS validation failed."
                exit_val=1
                ;;
            5)
                sg_log 0 "ERROR: Function reconfigure_start_function: Failed to restart PPAS monitoring service."
                exit_val=1
                ;;
            6)
                sg_log 0 "ERROR: Incorrect parameter(s) to toolkit script."
                ;;
            *)
                sg_log 0 "ERROR: Incorrect parameter(s) to function check_return."
                ;;
        esac
    fi
}

###############################################################################
# Function: ppas_dbi_start_cmd
#
# Start PPAS Instance if it is not running already.
# Return 0 on success otherwise return 1
###############################################################################
function ppas_dbi_start_cmd
{
    # See if the pg_ctl exist.
    if [[ ! -f $PPAS_DB_HOME/bin/pg_ctl ]]
    then
        sg_log 0 "ERROR: $PPAS_DB_HOME/bin/pg_ctl does not exist. Postgres Plus Advanced Server cannot be started."
        return 1
    fi

    # Check if a PPAS instance is already running on same PPAS_DB_DATA_DIR
    if [ -e "$PPAS_DB_DATA_DIR/postmaster.pid" ]
    then
        # Get the pid of the main edb-postgres instance process from the postmaster.pid file
        pidofpro=`$HEAD -n 1 $PPAS_DB_DATA_DIR/postmaster.pid`
        alive=`$PS -p $pidofpro | $GREP $pidofpro`
        if [ "x$alive" != "x" ]
        then
            sg_log 0 "Postgres Plus Advanced Server is already running on $PPAS_DB_DATA_DIR"
            return 0
        fi
    fi

    # Postgres Plus Advanced Server is not running already.
    sg_log 0 "Starting Postgres Plus Advanced Server:"

    $SU - $PPAS_DB_USER -c "$BASH" <<START
    export LD_LIBRARY_PATH=${PPAS_DB_HOME}/lib:\$LD_LIBRARY_PATH
    $PPAS_DB_HOME/bin/pg_ctl -w start -D $PPAS_DB_DATA_DIR -l $PPAS_DB_LOG_FILE
    exit \$?
START
    if [[ $? -eq 0 ]]; then
        sg_log 0 "Postgres Plus Advanced Server started successfully on $PPAS_DB_DATA_DIR"
    else
        sg_log 0 "ERROR: Postgres Plus Advanced Server did not start in time, refer PPAS DB log file for details."
        return 1
    fi

    return 0
}

###############################################################################
# Function: ppas_dbi_stop_cmd
#
# Shut down PPAS instance if it is running.
###############################################################################
function ppas_dbi_stop_cmd
{
    # See if the PPAS process is already stopped.
    $PS -aef | $GREP $PPAS_DB_HOME | $GREP $PPAS_DB_DATA_DIR >/dev/null 2>&1
    if [[ $? -ne 0 ]] ; then
        sg_log 0 "Postgres Plus Advanced Server is not running on $PPAS_DB_DATA_DIR"
        return 0
    fi

    # Postgres Plus Advanced Server is running.
    sg_log 0 "Stopping Postgres Plus Advanced Server:"
    $SU - $PPAS_DB_USER -c "$BASH" <<STOP
    export LD_LIBRARY_PATH=${PPAS_DB_HOME}/lib:\$LD_LIBRARY_PATH
    $PPAS_DB_HOME/bin/pg_ctl stop -m fast -w -D $PPAS_DB_DATA_DIR
STOP

    # See if the PPAS process is still running.
    $PS -aef | $GREP $PPAS_DB_HOME | $GREP $PPAS_DB_DATA_DIR >/dev/null 2>&1
    if [[ $? -eq 0 ]] ; then
        sg_log 0 "Graceful shutdown of Postgres Plus Advanced Server on $PPAS_DB_DATA_DIR failed. Killing the PPAS process."
        $PS -aef | $GREP $PPAS_DB_HOME | $GREP $PPAS_DB_DATA_DIR | $AWK '{print $2}' | xargs $KILL -9
    fi
    sg_log 0 "Postgres Plus Advanced Server stopped successfully on $PPAS_DB_DATA_DIR"

    return 0
}

#############################
#
# VALIDATE RECONFIGURE RESTART FUNCTION 
# Return
#        0 if service related package attribute is NOT modified.
#        1 if service related package attribute is modified.
#############################
function validate_reconfigure_restart
{
    if [[ $PPAS_MONITOR_INTERVAL = $PREV_PPAS_MONITOR_INTERVAL ]] &&
       [[ $SG_SCRIPT_LOG_FILE = $PREV_SG_SCRIPT_LOG_FILE ]] &&
       [[ ${SG_EMAIL_ID[*]} =  ${PREV_SG_EMAIL_ID[*]} ]]
    then
        return 0
    fi
    return 1
}

#############################
#
# RECONFIGURE STOP FUNCTION 
#
#############################
function reconfigure_stop_function
{
    if sg_check_if_reconfig_has_failed
    then
        sg_log 999 "WARNING: Not attempting to stop the EDB PPAS service $service_name for online package modification."
        return
    fi
    validate_reconfigure_restart
    if [[ $? -ne 0 ]]
    then
        if ! ${SGSBIN}/cmhaltserv ${service_name} ; then
           sg_log 999 "ERROR: Could not stop  the service $service_name for online package modification."
           sg_set_reconfig_failure
           exit_val=1
        else
           sg_log 999 "SUCCESS: Successfully stopped the service $service_name online package modification."
           if ! sg_check_if_reconfig_has_failed ; then  
                sg_log 0 "SUCCESS: Successfully stopped the service $service_name for online package modification."
           fi
        fi
    fi
}

#############################
#
# RECONFIGURE START FUNCTION 
#
# Return
#        0 if the service starts fine.
#        1 if the service fails to start.
#############################
function reconfigure_start_function
{   
    validate_reconfigure_restart
    if [[ $? -ne 0 ]]
    then
        msg="cmrunserv: Unable to start service $service_name (it is already active)"
        out=`${SGSBIN}/cmrunserv ${service_name} "${service_cmd_string}" 2>&1`
        exit_code=$? 
        if [[ $exit_code -ne 0 ]] && [[ "$msg" != "$out" ]] ;
        then
            sg_log 999 "ERROR: Failed to start the service $service_name for online package modification."
            sg_set_reconfig_failure 
            return 1 
        elif [[ $exit_code -eq 0 ]] ; then
            sg_log 999 "SUCCESS: Successfully started the service $service_name for online package modification."
            if ! sg_check_if_reconfig_has_failed ; then
                 sg_log 0 "SUCCESS: Succesfully started service $service_name for online package modification. "
            fi
        fi 
    fi
    return 0
}

#
# Identify the service name and service command for the package.
#
if [[ ${op_code} = "reconfigure_stop" || ${op_code} = "reconfigure_start" ]]
then
    if [[ ${#SG_SERVICE_CMD[@]} -ne 0 ]]
    then
        i=0
        service_name=""
        service_cmd_string=""
        for cmd in "${SG_SERVICE_CMD[@]}"
        do
            $ECHO "$cmd" | $GREP "ppas_monitor" >> /dev/null
            if [[ $? -eq 0 ]]
            then
                service_name=${SG_SERVICE_NAME[$i]}
                service_cmd_string="$cmd"
                break
            fi
            i=$i+1
        done
    fi
fi

case "${op_code}" in
    "start")
                if sg_check_if_reconfig_has_failed ; then
                    sg_log 999 "WARNING: Not attempting to start EDB PPAS module"
                    return 
                else
                    sg_log 0 "Starting PPAS Instance on DB data directory $PPAS_DB_DATA_DIR"
                    ppas_dbi_start_cmd
                    check_status=$? 
                    if [[ $check_status -ne 0 ]] ; then
                       if sg_check_if_it_is_reconfiguration ; then
                          sg_log 999 "ERROR: Failed to start EDB PPAS module"
                          sg_set_reconfig_failure
                       fi
                    else
                       if sg_check_if_it_is_reconfiguration ; then
                          sg_log 999 "SUCCESS: Successfully started EDB PPAS module"
                       fi
                    fi
                    check_return $check_status 1 
                fi
        ;;
    "stop")
        sg_log 0 "Stopping PPAS Instance on DB data directory $PPAS_DB_DATA_DIR"
        ppas_dbi_stop_cmd
        check_status=$?
        check_return $check_status 2
        ;;
    "ppas_monitor")
        sg_log 0 "Starting PPAS Monitoring on DB data directory $PPAS_DB_DATA_DIR"
        . $TKIT_SCRIPT_DIR/happas.mon
        check_status=$?
        check_return $check_status 3
        ;;
    "validate")
        validate_function
        check_status=$?
        check_return $check_status 4
        ;;
    "reconfigure_start")
        reconfigure_start_function
        check_status=$?
        check_return $check_status 5
        ;;
    "reconfigure_stop")
        reconfigure_stop_function
        ;;
    *)
        check_return 1 6
        sg_log 0 "Usage: ${0} [ start | stop | validate | ppas_monitor | reconfigure_start | reconfigure_stop ]"
        ;;
esac

exit $exit_val
